<?php

/**
 * @file
 * A Dependent Fields extension to allow referencing of existing users.
 *
 * The "hooks" in this file are not true hooks, they're called individually
 * from the main dependent_fields.module in the corresponding hook by the
 * same name. Any of these hooks could be broken out into a separate module.
 */

/**
 * Implements hook_dependent_fields_info().
 */
function dependent_fields_userreference_info() {
  $field = array();
  $field['userreference'] = array(
    'name' => t('User Reference'),
    'controller_widgets' => array(
      'userreference_select',
      'userreference_autocomplete',
      'userreference_buttons',
    ),
    'controlled_widgets' => array(
      'userreference_select',
      'userreference_autocomplete',
      'userreference_buttons',
    ),
  );
  return $field;
}

/**
 * Implements of hook_menu().
 */
function dependent_fields_userreference_menu() {
  $items = array();
  $items['dependent_fields/userreference/autocomplete/%/%/%'] = array(
    'page callback' => 'dependent_fields_userreference_autocomplete',
    'page arguments' => array(3, 4, 5),
    'access arguments' => array('access user profiles'),
    'file' => 'fields/userreference.inc',
    'type' => MENU_CALLBACK,
  );
  return $items;
}

/**
 * Implements hook_dependent_fields_field_form().
 */
function dependent_fields_userreference_field_form($field, $type) {
  $return = array();
  return $return;
}

/**
 * Implements hook_dependent_fields_widget_form().
 */
function dependent_fields_userreference_widget_form($widget, $field, $type) {
  $form = array();
  $filter_field = isset($widget['dependent_fields']['filter_field']) ? $widget['dependent_fields']['filter_field'] : '';
  $filter_forced = isset($widget['dependent_fields']['filter_forced']) ? $widget['dependent_fields']['filter_forced'] : TRUE;
  $controller_view_args = isset($widget['dependent_fields']['advanced']['controller_view_args']) ? $widget['dependent_fields']['advanced']['controller_view_args'] : '';
  $filter_fields = _dependent_fields_userreference_potential_filter_fields();

  $form['filter_field'] = array(
    '#type' => 'select',
    '#title' => t('Filter Field'),
    '#description' => t('Choose which field the potential user references will be filtered by.'),
    '#options' => $filter_fields,
    '#default_value' => $filter_field,
  );

  $form['filter_forced'] = array(
    '#type' => 'checkbox',
    '#title' => t('Force Controller Filter'),
    '#description' => t('When no controller field value has been set should the filter still be applied? If selected this will result in an empty list when no controller value is set.'),
    '#default_value' => $filter_forced,
  );

  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced - Override Views arguments'),
    '#collapsible' => TRUE,
    '#collapsed' => empty($controller_view_args),
  );

  $form['advanced']['controller_view_args'] = array(
    '#type' => 'textfield',
    '#title' => t('View arguments'),
    '#default_value' => $controller_view_args,
    '#required' => FALSE,
    '#description' => t('Provide a comma separated list of arguments to pass to the view.'),
  );

  $form['advanced']['controller_tokens_help'] = array(
    '#value' => dependent_fields_string_tokens_help(),
  );

  return $form;
}

/**
 * Implements hook_dependent_fields_widget_settings().
 */
function dependent_fields_userreference_widget_settings($op, $field) {
  $return = array();
  return $return;
}

/**
 * An #after_build callback for the field element
 */
function dependent_fields_userreference_element_after_build($element, $info, $form_state) {
  return $element;
}

/**
 * An #process callback for the userreference element
 */
function dependent_fields_userreference_element_process($element, $info, $form_state) {
  if (!empty($info['controller_field'])) {
    // Check if the widget is an autocomplete so we can change the path
    if (in_array($element['#type'], array('userreference_autocomplete'))) {
      // Get the list of controller uids
      $keys = dependent_fields_retrieve_field_keys($info['controller_field'], $info['controller_value']);
      // We can use the standard autocomplete path if we don't need to filter
      if (empty($keys)) {
        return $element;
      }
      $type_name = $info['field']['type_name'];
      $field_name = $info['field']['field_name'];
      $values = implode('-', $keys);
      // Build our new autocomplete path
      $path = "dependent_fields/userreference/autocomplete/$type_name/$field_name/$values";
      $element['uid']['#autocomplete_path'] = $path;
    }
  }

  return $element;
}

/**
 * An #validate callback for the userreference element
 */
function dependent_fields_userreference_element_validate($element, $data, $form_state) {
  // TODO: See if this handler is actually needed
  $field = $data['field'];
  $uids = dependent_fields_userreference_retrieve_field_keys($data['field'], $data['field_value']);
  // Don't check if there are no uids
  if ($uids) {
    $field_key = $element['#columns'][0];
    $references = _dependent_fields_userreference_potential_references($field, $data, '', NULL, $uids);
    foreach ($uids as $uid) {
      if (is_numeric($uid) && !isset($references[$uid])) {
        form_error($element, t("%name: this item can't be referenced .", array('%name' => t($field['widget']['label']))));
      }
    }
  }
  return $element;
}

/**
 * Implements hook_dependent_fields_retrieve_field_value().
 */
function dependent_fields_userreference_retrieve_field_value($field, $form_value, $form_state) {
  $return = array();
  if (empty($form_value)) {
    return $return;
  }

  $multiple = isset($form_value[0]);

  if ($multiple) {
    foreach ($form_value as $delta => $item) {
      if (!is_numeric($delta) || empty($item)) {
        continue;
      }
      $return[] = dependent_fields_userreference_retrieve_field_value($field, $item, $form_state);
    }
  }
  else {
    // Make sure we get the actual form element value.
    if (is_array($form_value['uid'])) {
      $value = $form_value['uid']['uid'];
    }
    else {
      $value = $form_value['uid'];
    }

    if (is_numeric($value)) {
      $return = array('uid' => $value);
    }
    elseif (is_array($value)) {
      foreach ($value as $uid) {
        $return[] = $uid;
      }
    }
    elseif (is_string($value) && !empty($value)) {
      // Parse the uid from the autocomplete string
      $uid = db_result(db_query("SELECT uid FROM {users} WHERE name = '%s'", $value));
      if (!empty($uid)) {
        $return = array(
          'uid' => $uid,
          'title' => $value,
        );
      }
      else {
        // No [uid:n] value matched, so we fallback to searching for the name.
        $handler = $form_state['dependent_handler'];
        if (!empty($handler)) {
          $data = $handler->get_field_data($field['field_name']);
          $reference = _dependent_fields_userreference_potential_references($field, $data, $value, 'equals', NULL, 1);
          if (!empty($reference)) {
            $uid = key($reference);
            $return = array(
              'uid' => $uid,
              'title' => $reference[$uid],
            );
          }
        }
      }
    }
  }

  return $return;
}

/**
 * Implements hook_dependent_fields_retrieve_field_keys().
 */
function dependent_fields_userreference_retrieve_field_keys($field, $values) {
  $return = array();
  if (empty($values)) {
    return;
  }
  foreach ($values as $value) {
    // Locate the uid for each value.
    if (is_array($value) && is_numeric($value['uid'])) {
      $return[] = $value['uid'];
    }
    elseif (is_numeric($value)) {
      $return[] = $value;
    }
  }
  return $return;
}

/**
 * Implements hook_dependent_fields_retrieve_field_titles().
 */
function dependent_fields_userreference_retrieve_field_titles($field, $values) {
  $return = array();
  foreach ($values as $value) {
    $uid = NULL;
    // Locate the title for each value.
    if (is_array($value) && !empty($value['title'])) {
      $return[] = $value['title'];
      continue;
    }
    elseif (is_array($value) && is_numeric($value['uid'])) {
      $uid = $value['uid'];
    }
    elseif (is_numeric($value)) {
      $uid = $value;
    }
    if (!empty($uid)) {
      $sql = 'SELECT name FROM {users} WHERE uid = %d';
      $result = db_query($sql, $uid);
      $title = db_result($result);
      if (!empty($title)) {
        $return[] = $title;
      }
    }
  }
  return $return;
}

/**
 * Implements hook_dependent_fields_build_field_info().
 */
function dependent_fields_userreference_build_field_info($info, $field, $widget, $type) {
  $return = array();
  $filter_field = $widget['dependent_fields']['filter_field'];
  $filter_forced = isset($widget['dependent_fields']['filter_forced']) ? $widget['dependent_fields']['filter_forced'] : TRUE;
  $controller_view_args = isset($widget['dependent_fields']['advanced']['controller_view_args']) ? $widget['dependent_fields']['advanced']['controller_view_args'] : '';

  if (!empty($filter_field)) {
    $return['filter_field'] = $filter_field;
  }

  $return['filter_forced'] = $filter_forced;
  $return['controller_view_args'] = $controller_view_args;
  return $return;
}

/**
 * Implements hook_allowed_values().
 */
function dependent_fields_userreference_allowed_values($field, $data) {
  $references = _dependent_fields_userreference_potential_references($field, $data);
  return $references;
}

/**
 * Menu callback; Retrieve a pipe delimited string of autocomplete suggestions
 * for existing users.
 */
function dependent_fields_userreference_autocomplete($type_name, $field_name, $controller_values, $string = '') {
  $field = content_fields($field_name, $type_name);
  $controller_field = dependent_fields_controller_field($field, $type_name);

  $controller_value = explode('-', $controller_values);
  $info = dependent_fields_build_field_info($field, NULL, $controller_value);

  $match = isset($field['widget']['autocomplete_match']) ? $field['widget']['autocomplete_match'] : 'contains';
  $matches = array();

  $references = _dependent_fields_userreference_potential_references($field, $data, $string, $match, array(), 10);
  foreach ($references as $id => $row) {
    // Add a class wrapper for a few required CSS overrides.
    $matches[$row['title']] = '<div class="reference-autocomplete">'. $row['rendered'] . '</div>';
  }
  drupal_json($matches);
}

/**
 * Fetch an array of all candidate referenced users.
 */
function _dependent_fields_userreference_potential_references($field, $data, $string = '', $match = 'contains', $ids = array(), $limit = NULL) {
  static $results = array();

  // Create unique id for static cache.
  $cid = $field['field_name'] . ':' . $match . ':' . ($string !== '' ? $string : implode('-', $ids)) . ':' . $limit;
  if (!isset($results[$cid])) {
    $references = FALSE;
    // Check if we need to use a view to get the potential references
    if (module_exists('views') && !empty($field['advanced_view']) && $field['advanced_view'] != '--') {
      $references = _dependent_fields_userreference_potential_references_views($field, $data, $string, $match, $ids, $limit);
    }
    // If the view doesn't exist, we got FALSE, and fallback to the regular 'standard mode'.

    if ($references === FALSE) {
      $references = _dependent_fields_userreference_potential_references_filtered($field, $data, $string, $match, $ids, $limit);
    }
    // Store the results.
    $results[$cid] = !empty($references) ? $references : array();
  }

  return $results[$cid];
}

function _dependent_fields_userreference_potential_references_views($field, $data, $string = '', $match = 'contains', $ids = array(), $limit = NULL) {
  $references = array();
  $info = $data['info'];
  if (!empty($info['controller_view_args'])) {
    $default_args = $field['advanced_view_args'];
    $view_args = $info['controller_view_args'];

    // Replace our tokens with controller values
    $view_args = dependent_fields_replace_string_tokens($data, $view_args);

    $field['advanced_view_args'] = $view_args;
    $references = _userreference_potential_references_views($field, $string, $match, $ids, $limit);
    $field['advanced_view_args'] = $default_args;
  }
  else {
    $references = _userreference_potential_references_views($field, $string, $match, $ids, $limit);
  }
  return $references;
}

/**
 * Helper function for _userreference_potential_references():
 * referenceable users in system.
 */
function _dependent_fields_userreference_potential_references_filtered($field, $data, $string = '', $match = 'contains', $ids = array(), $limit = NULL) {
  $where = array();
  $join = array();
  $args = array();

  $info = $data['info'];
  $controller = $data['controller'];

  // Filter by user roles and status that can be referenced.
  if (isset($field['referenceable_roles']) && is_array($field['referenceable_roles'])) {
    $rids = array(DRUPAL_ANONYMOUS_RID, DRUPAL_AUTHENTICATED_RID);
    $rids = array_diff(array_filter($field['referenceable_roles']), $rids);
    if (!empty($rids)) {
      $join[] = 'LEFT JOIN {users_roles} ur ON u.uid = ur.uid';
      $where[] = 'ur.rid IN (' . db_placeholders($rids) . ')';
      $args = array_merge($args, $rids);
    }
  }
  if (isset($field['referenceable_status']) && is_numeric($field['referenceable_status'])) {
    $where[] = 'u.status = %d';
    $args[] = $field['referenceable_status'];
  }

  // Check if there is a controller field, and if so filter the results
  // TODO: more options on how the filter is applied.
  if (!empty($controller)) {
    $filter_keys = $controller['keys'];
    $filter_field = !empty($info['filter_field']) ? $info['filter_field'] : '';
    // If the values are empty and force filter is set to true then return an
    // empty array.
    if (empty($filter_keys) && $info['filter_forced']) {
      return array();
    }
    elseif (!empty($filter_keys) && !empty($filter_field)) {
      $schema = drupal_get_schema('users');
      $type = $schema['fields'][$filter_field]['type'];
      // Use the filter field if it is set.
      $where[] = "u.$filter_field IN (" . db_placeholders($filter_keys, $type) . ")";
      $args = array_merge($args, $filter_keys);
    }
  }

  if ($string !== '') {
    $match_operators = array(
      'contains' => "LIKE '%%%s%%'",
      'equals' => "= '%s'",
      'starts_with' => "LIKE '%s%%'",
    );
    $where[] = 'u.name ' . (isset($match_operators[$match]) ? $match_operators[$match] : $match_operators['contains']);
    $args[] = $string;
  }
  elseif ($ids) {
    $where[] = 'u.uid IN (' . db_placeholders($ids) . ')';
    $args = array_merge($args, $ids);
  }
  else {
    $where[] = 'u.uid > 0';
  }

  $join_clause = $join ? implode(' ', $join) : '';
  $where_clause = $where ? 'WHERE ('. implode(') AND (', $where) .')' : '';
  $sql = "SELECT u.uid, u.name FROM {users} u $join_clause $where_clause ORDER BY u.name";
  $result = $limit ? db_query_range($sql, $args, 0, $limit) : db_query($sql, $args);
  $references = array();
  while ($user = db_fetch_object($result)) {
    $references[$user->uid] = array(
      'title' => $user->name,
      'rendered' => check_plain($user->name),
    );
  }

  return $references;
}

function _dependent_fields_userreference_potential_filter_fields($include_none = TRUE) {
  $options = array(
    'uid' => t('ID'),
    'name' => t('Name'),
    'mail' => t('E-mail address'),
    'status' => t('Status: 1 active, 0 blocked'),
    'language' => t('Language'),
    'signature' => t('Signature'),
  );

  if ($include_none) {
    array_unshift($options, t('No Filter Field'));
  }

  return $options;
}
